<?php

/*
 * This file is part of the Sylius package.
 *
 * (c) Paweł Jędrzejewski
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Sylius\Bundle\ContentBundle\Doctrine\ODM\PHPCR;

use Doctrine\ODM\PHPCR\DocumentManagerInterface;
use Doctrine\ODM\PHPCR\Mapping\ClassMetadata;
use Sylius\Bundle\ContentBundle\Document\StaticContent;
use Sylius\Bundle\ContentBundle\Repository\StaticContentRepositoryInterface;
use Sylius\Bundle\ResourceBundle\Doctrine\ODM\PHPCR\DocumentRepository;

/**
 * @author Paweł Jędrzejewski <pawel@sylius.org>
 */
class StaticContentRepository extends DocumentRepository implements StaticContentRepositoryInterface
{
    /**
     * @var string
     */
    private $staticContentPath;

    /**
     * @param DocumentManagerInterface $dm
     * @param ClassMetadata $class
     * @param string $staticContentPath
     */
    public function __construct(DocumentManagerInterface $dm, ClassMetadata $class, $staticContentPath)
    {
        parent::__construct($dm, $class);

        $this->staticContentPath = $staticContentPath;
    }

    /**
     * {@inheritdoc}
     */
    public function findPublishedOneByName($name)
    {
        /** @var StaticContent|null $staticContent */
        $staticContent = $this->find($this->staticContentPath . '/' . $name);

        if (null === $staticContent || !$staticContent->isPublishable()) {
            return null;
        }

        return $staticContent;
    }
}
